@@ -18,8 +18,8 @@ int		CTRL_HOME;
 int		CTRL_END;
 int		CTRL_SHIFT_HOME;
 int		CTRL_SHIFT_END;
-
-#ifdef NCURSES_EXT_FUNCS
+int		CTRL_LEFT;
+int		CTRL_RIGHT;
 
 static int
 get_code(const char *capname, int fallback)
@@ -46,8 +46,6 @@ get_code(const char *capname, int fallback)
 
 }
 
-#endif
-
 /*
  * Set a value of CTRL_HOME and CTRL_END key codes. These codes
  * can be redefined on some plaforms.
@@ -66,6 +64,8 @@ initialize_special_keycodes()
 	CTRL_END = get_code("kEND5", 533);
 	CTRL_SHIFT_HOME = get_code("kHOM6", 537);
 	CTRL_SHIFT_END = get_code("kEND6", 532);
+	CTRL_RIGHT = get_code("kRIT5", 561);
+	CTRL_LEFT = get_code("kLFT5", 546);
 }
 
 /*
@@ -174,6 +174,8 @@ cmd_string(int cmd)
 			return "SetTheme_PaperColor";
 		case cmd_SetTheme:
 			return "SetTheme";
+		case cmd_SetCustomTheme:
+			return "SetCustomTheme";
 
 		case cmd_SaveSetup:
 			return "SaveSetup";
@@ -210,6 +212,10 @@ cmd_string(int cmd)
 			return "MoveLeft";
 		case cmd_MoveRight:
 			return "MoveRight";
+		case cmd_MoveCharLeft:
+			return "MoveCharLeft";
+		case cmd_MoveCharRight:
+			return "MoveCharRight";
 		case cmd_CursorFirstRow:
 			return "CursorFirstRow";
 		case cmd_CursorLastRow:
@@ -340,6 +346,13 @@ cmd_string(int cmd)
 
 		case cmd_BsCommand:
 			return "BsCommand";
+		case cmd_ShowPrimaryScreen:
+			return "ShowPrimaryScreen";
+
+		case cmd_ToggleHighlightOddRec:
+			return "ToggleHighlightOddRec";
+		case cmd_ToggleHideHeaderLine:
+			return "ToggleHideHeaderLine";
 
 		default:
 			return "unknown command";
@@ -514,6 +527,8 @@ translate_event(int c, bool alt, Options *opts, int *nested_command)
 			case 'R':
 			case 12:	/* CTRL L */
 				return cmd_Refresh;
+			case 15:	/* CTRL O */
+				return cmd_ShowPrimaryScreen;
 
 			case '\\':
 				return cmd_BsCommand;
@@ -547,6 +562,10 @@ translate_event(int c, bool alt, Options *opts, int *nested_command)
 		*nested_command = cmd_CursorFirstRow;
 		return cmd_Mark_NestedCursorCommand;
 	}
+	else if (c == CTRL_LEFT)
+		return cmd_MoveCharLeft;
+	else if (c == CTRL_RIGHT)
+		return cmd_MoveCharRight;
 
 	return cmd_Invalid;
 }
@@ -674,3 +693,27 @@ cmd_get_theme(int cmd)
 
 	return 1;
 }
+
+/*
+ * This is initial simple design. It can be much
+ * smarter, but it is good enough for start.
+ */
+bool
+require_complete_load(int cmd)
+{
+	switch (cmd)
+	{
+		case cmd_CursorLastRow:
+		case cmd_SortAsc:
+		case cmd_SortDesc:
+		case cmd_OriginalSort:
+		case cmd_SaveData:
+		case cmd_Copy:
+		case cmd_GotoLine:
+		case cmd_ForwardSearch:
+			return true;
+
+		default:
+			return false;
+	}
+}
