@@ -71,6 +71,7 @@ typedef enum PspgCommand
 	cmd_SetTheme_RelationalPipes,
 	cmd_SetTheme_PaperColor,
 	cmd_SetTheme,
+	cmd_SetCustomTheme,
 	cmd_SaveSetup,
 
 	cmd_Escape,
@@ -89,6 +90,8 @@ typedef enum PspgCommand
 	cmd_ScrollDownHalfPage,
 	cmd_MoveLeft,
 	cmd_MoveRight,
+	cmd_MoveCharLeft,
+	cmd_MoveCharRight,
 	cmd_CursorFirstRow,
 	cmd_CursorLastRow,
 	cmd_CursorFirstRowPage,
@@ -151,12 +154,16 @@ typedef enum PspgCommand
 	cmd_Unmark,
 	cmd_Mark_NestedCursorCommand,
 
-	cmd_BsCommand
+	cmd_BsCommand,
+	cmd_ShowPrimaryScreen,
+	cmd_ToggleHighlightOddRec,
+	cmd_ToggleHideHeaderLine,
 } PspgCommand;
 
 extern void initialize_special_keycodes();
 extern const char *cmd_string(int cmd);
 extern int translate_event(int c, bool alt, Options *opts, int *nested_command);
+extern bool require_complete_load(int cmd);
 
 extern bool is_cmd_RowNumToggle(int c, bool alt);
 
