@@ -14,10 +14,12 @@
 #include "config.h"
 
 #include <errno.h>
+#include <limits.h>
 #include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <sys/stat.h>
 
 static int
 parse_cfg(char *line, char *key, bool *bool_val, int *int_val, char **str_val)
@@ -61,7 +63,7 @@ parse_cfg(char *line, char *key, bool *bool_val, int *int_val, char **str_val)
 		while (*line == ' ')
 			line++;
 
-		if (*line >= '0' && *line <= '9')
+		if (*line == '-' || (*line >= '0' && *line <= '9'))
 		{
 			*int_val = atoi(line);
 			return 1;
@@ -83,8 +85,7 @@ parse_cfg(char *line, char *key, bool *bool_val, int *int_val, char **str_val)
 
 			size = strlen(line);
 			str = trim_quoted_str(line, &size);
-
-			*str_val = sstrndup(str, size);
+			*str_val = str ? sstrndup(str, size) : NULL;
 			return 3;
 		}
 	}
@@ -113,6 +114,9 @@ save_config(char *path, Options *opts)
 	if (f == NULL)
 		return false;
 
+	if (chmod(path, 0644) != 0)
+		return false;
+
 	SAFE_SAVE_BOOL_OPTION("ascii_menu", opts->force_ascii_art);
 	SAFE_SAVE_BOOL_OPTION("bold_labels", opts->bold_labels);
 	SAFE_SAVE_BOOL_OPTION("bold_cursor", opts->bold_cursor);
@@ -139,6 +143,9 @@ save_config(char *path, Options *opts)
 	SAFE_SAVE_BOOL_OPTION("menu_always", opts->menu_always);
 	SAFE_SAVE_BOOL_OPTION("empty_string_is_null", opts->empty_string_is_null);
 	SAFE_SAVE_BOOL_OPTION("last_row_search", opts->last_row_search);
+	SAFE_SAVE_BOOL_OPTION("progressive_load_mode", opts->progressive_load_mode);
+	SAFE_SAVE_BOOL_OPTION("highlight_odd_rec", opts->highlight_odd_rec);
+	SAFE_SAVE_BOOL_OPTION("hide_header_line", opts->hide_header_line);
 
 	result = fprintf(f, "theme = %d\n", opts->theme);
 	if (result < 0)
@@ -156,6 +163,10 @@ save_config(char *path, Options *opts)
 	if (result < 0)
 		return false;
 
+	result = fprintf(f, "hist_size = %d\n", opts->hist_size);
+	if (result < 0)
+		return false;
+
 	if (opts->nullstr)
 	{
 		result = fprintf(f, "nullstr = \"%s\"\n", opts->nullstr);
@@ -163,6 +174,13 @@ save_config(char *path, Options *opts)
 			return false;
 	}
 
+	if (opts->custom_theme_name)
+	{
+		result = fprintf(f, "custom_theme_name = \"%s\"\n", opts->custom_theme_name);
+		if (result < 0)
+			return false;
+	}
+
 	result = fclose(f);
 	if (result != 0)
 		return false;
@@ -308,15 +326,24 @@ load_config(char *path, Options *opts)
 				is_valid = assign_bool(key, &opts->empty_string_is_null, bool_val, res);
 			else if (strcmp(key, "last_row_search") == 0)
 				is_valid = assign_bool(key, &opts->last_row_search, bool_val, res);
-
-			free(line);
-			line = NULL;
+			else if (strcmp(key, "hist_size") == 0)
+				is_valid = assign_int(key, (int *) &opts->hist_size, int_val, res, 0, INT_MAX);
+			else if (strcmp(key, "progressive_load_mode") == 0)
+				is_valid = assign_bool(key, &opts->progressive_load_mode, bool_val, res);
+			else if (strcmp(key, "custom_theme_name") == 0)
+				is_valid = assign_str(key, &opts->custom_theme_name, str_val, res);
+			else if (strcmp(key, "highlight_odd_rec") == 0)
+				is_valid = assign_bool(key, &opts->highlight_odd_rec, bool_val, res);
+			else if (strcmp(key, "hide_header_line") == 0)
+				is_valid = assign_bool(key, &opts->hide_header_line, bool_val, res);
 
 			if (!is_valid || res == -1)
 				break;
 		}
 	}
 
+	free(line);
+
 	fclose(f);
 
 	return is_valid;
