@@ -92,6 +92,11 @@ typedef struct
 	bool	querystream;
 	bool	menu_always;
 	bool	last_row_search;
+	int		hist_size;
+	bool	progressive_load_mode;
+	char   *custom_theme_name;
+	bool	highlight_odd_rec;
+	bool	hide_header_line;
 } Options;
 
 extern bool save_config(char *path, Options *opts);
