@@ -13,6 +13,7 @@
 
 #include "pspg.h"
 
+#include <limits.h>
 #include <stdlib.h>
 
 /*
@@ -181,13 +182,36 @@ lbm_xor_mask(LineBufferMark *lbm, char mask)
 {
 	if (!lbm->lb->lineinfo)
 	{
+		int		i;
+
 		/* smalloc returns zero fill memory already */
 		lbm->lb->lineinfo = smalloc(LINEBUFFER_LINES * sizeof(LineInfo));
+
+		for (i = 0; i < LINEBUFFER_LINES; i++)
+			lbm->lb->lineinfo[i].recno_offset = SHRT_MIN;
 	}
 
 	lbm->lb->lineinfo[lbm->lb_rowno].mask ^= mask;
 }
 
+void
+lbm_recno_offset(LineBufferMark *lbm, short int recno_offset)
+{
+	if (!lbm->lb->lineinfo)
+	{
+		int		i;
+
+		/* smalloc returns zero fill memory already */
+		lbm->lb->lineinfo = smalloc(LINEBUFFER_LINES * sizeof(LineInfo));
+
+		for (i = 0; i < LINEBUFFER_LINES; i++)
+			lbm->lb->lineinfo[i].recno_offset = SHRT_MIN;
+	}
+
+	lbm->lb->lineinfo[lbm->lb_rowno].recno_offset = recno_offset;
+}
+
+
 /*
  * Working horse of lbm_get_line and lbi_get_line routines
  */
@@ -509,3 +533,16 @@ lb_print_all_ddesc(DataDesc *desc, FILE *f)
 			break;
 	}
 }
+
+const char *
+getline_ddesc(DataDesc *desc, int pos)
+{
+	LineBufferIter lbi;
+	char	   *result;
+
+	init_lbi_ddesc(&lbi, desc, pos);
+	if (lbi_get_line(&lbi, &result, NULL, NULL))
+		return result;
+
+	return NULL;
+}
