@@ -23,6 +23,7 @@
 
 ST_CMDBAR_ITEM _bottombar[] = {
 	{"Save", false, 2, cmd_SaveData, 0},
+	{"Mark", false, 3, cmd_Mark, 0},
 	{"Search", false, 7, cmd_ForwardSearch, 0},
 	{"Menu", false, 9, cmd_ShowMenu, 0},
 	{"Quit", false, 10, cmd_Quit, 0},
@@ -45,23 +46,27 @@ ST_MENU_ITEM _copy[] = {
 	{"Copy ~l~ine", cmd_CopyLine, NULL, 0, 0, 0, NULL},
 	{"Copy line e~x~tended", cmd_CopyLineExtended, NULL, 0, 0, 0, NULL},
 	{"Copy col~u~mn", cmd_CopyColumn, NULL, 0, 0, 0, NULL},
+	{"Copy ~s~elected", cmd_CopySelected, NULL, 0, 0, 0, NULL},
 	{"--", 0, NULL, 0, 0, 0, NULL},
 	{"Copy ~a~ll", cmd_CopyAllLines, NULL, 0, 0, 0, NULL},
 	{"Copy ~t~op lines", cmd_CopyTopLines, NULL, 0, 0, 0, NULL},
 	{"Copy ~b~ottom lines", cmd_CopyBottomLines, NULL, 0, 0, 0, NULL},
-	{"Copy ~m~arked lines", cmd_CopyMarkedLines, NULL, 0, 0, 0, NULL},
-	{"Copy ~s~earched lines", cmd_CopySearchedLines, NULL, 0, 0, 0, NULL},
+	{"Copy book~m~arked lines", cmd_CopyMarkedLines, NULL, 0, 0, 0, NULL},
+	{"Copy sea~r~ched lines", cmd_CopySearchedLines, NULL, 0, 0, 0, NULL},
 	{"--", 0, NULL, 0, 0, 0, NULL},
 	{"Copy to ~f~ile", cmd_SetCopyFile, NULL, 0, 0, 0, NULL},
 	{"Copy to cli~p~board", cmd_SetCopyClipboard, NULL, 0, 0, 0, NULL},
 	{"--", 0, NULL, 0, 0, 0, NULL},
 	{"Empty string is NULL", cmd_TogleEmptyStringIsNULL, NULL, 0, 0, 0, NULL},
+	{"Set own ~N~ULL string", cmd_SetOwnNULLString, NULL, 0, 0, 0, NULL},
 	{"--", 0, NULL, 0, 0, 0, NULL},
 	{"_0_Use CSV format", cmd_UseClipboard_CSV, NULL, 0, 0, 0, NULL},
 	{"_1_Use LibreOffice TSVC format", cmd_UseClipboard_TSVC, NULL, 0, 0, 0, NULL},
 	{"_2_Use formatted text", cmd_UseClipboard_text, NULL, 0, 0, 0, NULL},
 	{"_3_Use INSERT format", cmd_UseClipboard_INSERT, NULL, 0, 0, 0, NULL},
 	{"_4_Use commented INSERT format", cmd_UseClipboard_INSERT_with_comments, NULL, 0, 0, 0, NULL},
+	{"_5_Use SQL Values format", cmd_UseClipboard_SQL_values, NULL, 0, 0, 0, NULL},
+	{"_6_Use pipe separated text", cmd_UseClipboard_pipe_separated, NULL, 0, 0, 0, NULL},
 	{NULL, 0, NULL, 0, 0, 0, NULL}
 };
 
@@ -86,6 +91,9 @@ ST_MENU_ITEM _search[] = {
 	{"Search ~a~gain", cmd_SearchNext, "n", 0, 0, 0, NULL},
 	{"Search p~r~evious", cmd_SearchPrev, "N", 0, 0, 0, NULL},
 	{"--", 0, NULL, 0, 0, 0, NULL},
+	{"Search in selection", cmd_ForwardSearchInSelection, "M-/", 0, 0, 0, NULL},
+	{"Search back in selection", cmd_BackwardSearchInSelection, "M-?", 0, 0, 0, NULL},
+	{"--", 0, NULL, 0, 0, 0, NULL},
 	{"Search ~c~olumn", cmd_SearchColumn, "c", 0, 0, 0, NULL},
 	{"--", 0, NULL, 0, 0, 0, NULL},
 	{"~T~oggle bookmark", cmd_ToggleBookmark, "M-k", 0, 0, 0, NULL},
@@ -121,7 +129,13 @@ ST_MENU_ITEM _command[] = {
 	{"~D~escending order", cmd_SortDesc, "d", 0, 0, 0, NULL},
 	{"~O~riginal order", cmd_OriginalSort, "u", 0, 0, 0, NULL},
 	{"--", 0, NULL, 0, 0, 0, NULL},
+	{"To~g~gle mark", cmd_Mark, "F3", 0, 0, 0, NULL},
+	{"~M~ark column", cmd_MarkColumn, "F13", 0, 0, 0, NULL},
+	{"Mark all", cmd_MarkAll, NULL, 0, 0, 0, NULL},
+	{"Unmar~k~", cmd_Unmark, NULL, 0, 0, 0, NULL},
+	{"--", 0, NULL, 0, 0, 0, NULL},
 	{"Refres~h~ screen", cmd_Refresh, "R, C-l", 0, 0, 0, NULL},
+	{"Show primar~y~ screen", cmd_ShowPrimaryScreen, "C-o", 0, 0, 0, NULL},
 	{NULL, 0, NULL, 0, 0, 0, NULL}
 };
 
@@ -149,6 +163,7 @@ ST_MENU_ITEM _theme[] = {
 	{"_t_Tao Light theme", cmd_SetTheme_TaoLight, NULL, 0, 0, 0, NULL},
 	{"_f_Flatwhite theme", cmd_SetTheme_Flatwhite, NULL, 0, 0, 0, NULL},
 	{"_a_Relational Pipes theme", cmd_SetTheme_RelationalPipes, NULL, 0, 0, 0, NULL},
+	{"_h_PaperColor theme", cmd_SetTheme_PaperColor, NULL, 0, 0, 0, NULL},
 	{NULL},
 };
 
@@ -166,6 +181,7 @@ ST_MENU_ITEM _options[] = {
 	{"Show line ~n~umbers", cmd_RowNumToggle, "M-n", 0, 0, 0, NULL},
 	{"Show top bar", cmd_ShowTopBar, NULL, 0, 0, 0, NULL},
 	{"Show bottom bar", cmd_ShowBottomBar, NULL, 0, 0, 0, NULL},
+	{"Show scrollbar", cmd_ShowScrollbar, NULL, 0, 0, 0, NULL},
 	{"--", 0, NULL, 0, 0, 0, NULL},
 	{"~M~ouse support", cmd_MouseToggle, "M-m", 0, 0, 0, NULL},
 	{"~Q~uiet mode", cmd_SoundToggle, NULL, 0, 0, 0, NULL},
@@ -174,7 +190,10 @@ ST_MENU_ITEM _options[] = {
 	{"Force ~a~scii menu", cmd_MenuAsciiArtToggle, NULL, 0, 0, 0, NULL},
 	{"Bold labels", cmd_BoldLabelsToggle, "M-b", 0, 0, 0, NULL},
 	{"Bold cursor", cmd_BoldCursorToggle, NULL, 0, 0, 0, NULL},
+	{"Hide header line", cmd_ToggleHideHeaderLine, NULL, 0, 0, 0, NULL},
+	{"Highlight odd records", cmd_ToggleHighlightOddRec, NULL, 0, 0, 0, NULL},
 	{"~T~heme", MENU_ITEM_THEME, NULL, 0, 0,  0, _theme},
+	{"S~e~t custom theme", cmd_SetCustomTheme, NULL, 0, 0, 0, NULL},
 	{"--", 0, NULL, 0, 0, 0, NULL},
 	{"~S~ave setup", cmd_SaveSetup, NULL, 0, 0, 0, NULL},
 	{NULL},
@@ -198,6 +217,12 @@ int menu_theme = -1;
 static int
 get_menu_style(int main_theme)
 {
+	if (current_state && current_state->menu_template >= 0)
+	{
+		log_row("use custom menu template %d", current_state->menu_template);
+		return current_state->menu_template;
+	}
+
 	switch (main_theme)
 	{
 		case 0:
@@ -240,6 +265,8 @@ get_menu_style(int main_theme)
 			return ST_MENU_STYLE_FLATWHITE;
 		case 22:
 			return ST_MENU_STYLE_DBASE;
+		case 23:
+			return ST_MENU_STYLE_PERFECT;
 		default:
 			return ST_MENU_STYLE_VISION;
 	}
@@ -251,7 +278,7 @@ get_menu_style(int main_theme)
 void
 init_menu_config(Options *opts)
 {
-	int start_from_rgb = 220;
+	int start_from_rgb = 190;
 
 	menu_theme = get_menu_style(opts->theme);
 
@@ -262,21 +289,21 @@ init_menu_config(Options *opts)
 		fcp = st_menu_load_style(&menu_config,
 								 menu_theme,
 								 30,
-								 opts->force8bit,
+								 !use_utf8,
 								 opts->force_ascii_art);
 
 		st_menu_load_style(&menu_config2,
 						   ST_MENU_STYLE_FREE_DOS_P,
 						   fcp,
-						   opts->force8bit,
+						   !use_utf8,
 						   opts->force_ascii_art);
 	}
 	else
 		st_menu_load_style_rgb(&menu_config,
 							   menu_theme,
-							   menu_theme == ST_MENU_STYLE_ONECOLOR ? 3 : 30,
+							   menu_theme == ST_MENU_STYLE_ONECOLOR ? 1 : 50,
 							   &start_from_rgb,
-							   opts->force8bit,
+							   !use_utf8,
 							   opts->force_ascii_art);
 
 	/* extra pspg menu theme customization */
@@ -290,7 +317,7 @@ init_menu_config(Options *opts)
  * Prepare configuration and initialize menu
  */
 struct ST_MENU *
-init_menu(struct ST_MENU *current_menu)
+init_menu(struct ST_MENU *current_menu, Options *opts)
 {
 	struct ST_MENU		*menu = NULL;
 
@@ -304,11 +331,16 @@ init_menu(struct ST_MENU *current_menu)
 		int		positions[1024];
 		int		*refvals[1024];
 
+		/* Save state of old menu, and load it to new menu */
 		st_menu_save(current_menu, positions, refvals, 1023);
 		st_menu_load(menu, positions, refvals);
 		st_menu_free(current_menu);
+		log_row("releasing menu");
 	}
 
+	if (opts->quit_on_f3)
+		st_menu_set_shortcut(menu, cmd_Mark, NULL);
+
 	return menu;
 }
 
@@ -322,10 +354,16 @@ init_cmdbar(struct ST_CMDBAR *current_cmdbar, Options *opts)
 	else
 		cmdbar = st_cmdbar_new(&menu_config, _bottombar);
 
+	/*
+	 * It looks obscure - it uses same pattern like init_menu
+	 * although the cmdbar has not state, so old cmdbar can
+	 * be released fully before creating new cmdbar.
+	 */
 	if (current_cmdbar)
 	{
 		st_cmdbar_unpost(current_cmdbar);
 		st_cmdbar_free(current_cmdbar);
+		log_row("releasing cmd bar");
 	}
 
 	return cmdbar;
@@ -341,7 +379,7 @@ post_menu(Options *opts, struct ST_MENU *menu)
 	st_menu_set_option(menu, cmd_FreezeThreeCols, ST_MENU_OPTION_MARKED, opts->freezed_cols == 3);
 	st_menu_set_option(menu, cmd_FreezeFourCols, ST_MENU_OPTION_MARKED, opts->freezed_cols == 4);
 
-	st_menu_set_option(menu, cmd_SoundToggle, ST_MENU_OPTION_MARKED, opts->no_sound);
+	st_menu_set_option(menu, cmd_SoundToggle, ST_MENU_OPTION_MARKED, quiet_mode);
 	st_menu_set_option(menu, cmd_UtfArtToggle, ST_MENU_OPTION_MARKED, opts->force_uniborder);
 	st_menu_set_option(menu, cmd_MenuAsciiArtToggle, ST_MENU_OPTION_MARKED, opts->force_ascii_art);
 	st_menu_set_option(menu, cmd_MouseToggle, ST_MENU_OPTION_MARKED, !opts->no_mouse);
@@ -366,6 +404,8 @@ post_menu(Options *opts, struct ST_MENU *menu)
 	st_menu_set_option(menu, cmd_BoldLabelsToggle, ST_MENU_OPTION_MARKED, opts->bold_labels);
 	st_menu_set_option(menu, cmd_BoldCursorToggle, ST_MENU_OPTION_MARKED, opts->bold_cursor);
 
+	st_menu_set_option(menu, cmd_ShowScrollbar, ST_MENU_OPTION_MARKED, opts->show_scrollbar);
+
 	st_menu_reset_all_submenu_options(menu, MENU_ITEM_THEME, ST_MENU_OPTION_MARKED);
 	st_menu_enable_option(menu, theme_get_cmd(opts->theme), ST_MENU_OPTION_MARKED);
 
@@ -373,6 +413,13 @@ post_menu(Options *opts, struct ST_MENU *menu)
 	refresh_clipboard_options(opts, menu);
 
 	st_menu_set_option(menu, cmd_TogleEmptyStringIsNULL, ST_MENU_OPTION_MARKED, opts->empty_string_is_null);
+
+	st_menu_set_option(menu, cmd_SetOwnNULLString, ST_MENU_OPTION_MARKED,
+			  !opts->empty_string_is_null && opts->nullstr && *opts->nullstr);
+
+	st_menu_set_option(menu, cmd_ToggleHighlightOddRec, ST_MENU_OPTION_MARKED, opts->highlight_odd_rec);
+	st_menu_set_option(menu, cmd_ToggleHideHeaderLine, ST_MENU_OPTION_MARKED, opts->hide_header_line);
+
 }
 
 void
@@ -384,6 +431,9 @@ refresh_clipboard_options(Options *opts, struct ST_MENU *menu)
 	st_menu_set_option(menu, cmd_UseClipboard_INSERT, ST_MENU_OPTION_MARKED, opts->clipboard_format == CLIPBOARD_FORMAT_INSERT);
 	st_menu_set_option(menu, cmd_UseClipboard_INSERT_with_comments, ST_MENU_OPTION_MARKED,
 					   opts->clipboard_format == CLIPBOARD_FORMAT_INSERT_WITH_COMMENTS);
+
+	st_menu_set_option(menu, cmd_UseClipboard_SQL_values, ST_MENU_OPTION_MARKED, opts->clipboard_format == CLIPBOARD_FORMAT_SQL_VALUES);
+	st_menu_set_option(menu, cmd_UseClipboard_pipe_separated, ST_MENU_OPTION_MARKED, opts->clipboard_format == CLIPBOARD_FORMAT_PIPE_SEPARATED);
 }
 
 void
