@@ -129,12 +129,6 @@ field_info(char *str, bool *multiline)
 		return utf_string_dsplen_multiline(str, strlen(str), multiline, false, &digits, &others);
 }
 
-static int
-max_int(int a, int b)
-{
-	return a > b ? a : b;
-}
-
 /*
  * Returns true, when some column should be hidden.
  */
@@ -410,6 +404,7 @@ pg_exec_query(Options *opts, char *query, RowBucketType *rb, PrintDataDesc *pdes
 
 	(void) rb;
 	(void) pdesc;
+	(void) opts;
 
 	*err = "Query cannot be executed. The Postgres library was not available at compile time.";
 
