@@ -20,6 +20,7 @@
 #include <string.h>
 #include <stdint.h>
 
+#include "inputs.h"
 #include "pspg.h"
 #include "unicode.h"
 
@@ -1366,6 +1367,7 @@ read_and_format(Options *opts, DataDesc *desc, StateData *state)
 	PrintbufType	printbuf;
 	PrintDataDesc	pdesc;
 	char	   *query = opts->query;
+	char	   *name;
 
 	state->errstr = NULL;
 	state->_errno = 0;
@@ -1399,11 +1401,9 @@ read_and_format(Options *opts, DataDesc *desc, StateData *state)
 
 	memset(desc, 0, sizeof(DataDesc));
 
-	if (state->pathname[0])
+	name = (char *) get_input_file_basename();
+	if ((name = (char *) get_input_file_basename()))
 	{
-		char	   *name;
-
-		name = basename(state->pathname);
 		strncpy(desc->filename, name, 64);
 		desc->filename[64] = '\0';
 	}
@@ -1430,6 +1430,12 @@ read_and_format(Options *opts, DataDesc *desc, StateData *state)
 	desc->maxbytes = -1;
 	desc->maxx = -1;
 
+	/*
+	 * This format doesn't support progressive load
+	 */
+	desc->initialized = true;
+	desc->completed = true;
+
 	memset(&desc->rows, 0, sizeof(LineBuffer));
 	desc->rows.prev = NULL;
 
@@ -1445,8 +1451,11 @@ read_and_format(Options *opts, DataDesc *desc, StateData *state)
 	pconfig.header_mode = opts->csv_header;
 	pconfig.ignore_short_rows = opts->ignore_short_rows;
 
+	memset(&rowbuckets, 0, sizeof(RowBucketType));
+
 	rowbuckets.allocated = false;
 	rowbuckets.nrows = 0;
+	rowbuckets.next_bucket = NULL;
 
 	if (query)
 	{
@@ -1461,24 +1470,42 @@ read_and_format(Options *opts, DataDesc *desc, StateData *state)
 			if (query != opts->query)
 				free(query);
 
+			free(linebuf.buffer);
+
 			return false;
 		}
 	}
 	else if (opts->csv_format)
 	{
+		if (!f_data)
+		{
+			format_error("missing data");
+			free(linebuf.buffer);
+
+			return false;
+		}
+
 		read_csv(&rowbuckets,
 				 &linebuf,
 				 opts->csv_separator,
-				 state->fp, opts->ignore_short_rows,
+				 f_data, opts->ignore_short_rows,
 				 opts);
 
 		prepare_pdesc(&rowbuckets, &linebuf, &pdesc, &pconfig);
 	}
 	else if (opts->tsv_format)
 	{
+		if (!f_data)
+		{
+			format_error("missing data");
+			free(linebuf.buffer);
+
+			return false;
+		}
+
 		read_tsv(&rowbuckets,
 				 &linebuf,
-				 state->fp,
+				 f_data,
 				 opts->ignore_short_rows,
 				 opts);
 
@@ -1533,7 +1560,6 @@ read_and_format(Options *opts, DataDesc *desc, StateData *state)
 			desc->last_row = desc->total_rows - 1;
 
 			desc->footer_row = desc->last_row;
-			desc->footer_rows = 1;
 
 			if (pconfig.border == 2)
 			{
@@ -1629,7 +1655,6 @@ read_and_format(Options *opts, DataDesc *desc, StateData *state)
 		desc->last_row = desc->total_rows - 1;
 
 		desc->footer_row = desc->last_row;
-		desc->footer_rows = 1;
 
 		if (pconfig.border == 2)
 		{
