@@ -65,22 +65,30 @@ static char	   *readline_prompt;
 static int		tabcomplete_mode;
 static const char **possible_tokens = NULL;
 
+static bool		readline_initized = false;
+
 #ifdef HAVE_READLINE_HISTORY
 
 static char		last_history[256];
 
+static const char *saved_histfile = NULL;
+static bool		history_loaded = false;
+
 #endif
 
 static const char *bscommands[] = {
 	"save",
 	"copy",
 	"theme",
+	"ctheme",
 	"quit",
 	"order",
 	"orderd",
 	"search",
 	"sort",
 	"sortd",
+	"rsort",
+	"dsort",
 	NULL
 };
 
@@ -114,14 +122,29 @@ static const char *search_opts[] = {
  * Save history to hist file
  */
 void
-pspg_save_history(const char *histfile)
+pspg_save_history(const char *histfile, Options *opts)
 {
 
 #ifdef HAVE_READLINE_HISTORY
 
-	write_history(tilde(NULL, histfile));
+	if (history_loaded)
+	{
+		if (opts->hist_size >= 0)
+			stifle_history(opts->hist_size);
 
-	rl_clear_history();
+		write_history(tilde(NULL, histfile));
+
+#if RL_READLINE_VERSION >= 0x0603
+
+		rl_clear_history();
+
+#else
+
+		clear_history();
+
+#endif
+
+	}
 
 #endif
 
@@ -146,6 +169,22 @@ readline_getc(FILE *dummy)
 {
 	UNUSED(dummy);
 
+#if RL_READLINE_VERSION < 0x0603
+
+	/*
+	 * readline uses this routine for reading one char. When
+	 * rl_input_available_hook is not available, readline try
+	 * to read multibyte char by repeated call of readline_getc,
+	 * until multibyte char is completed or this function returns
+	 * EOF. In this case, probably the most simple solution
+	 * is direct reading from input stream. These old versions
+	 * of readline can be on RH7 and Solaris.
+	 */
+	if (!readline_ncurses_proxy_char_is_available)
+		return wgetch(prompt_window);
+
+#endif
+
     readline_ncurses_proxy_char_is_available = false;
     return readline_ncurses_proxy_char;
 }
@@ -358,7 +397,10 @@ pspg_complete(const char *text, int start, int end)
 									 IS_TOKEN(token, n, "ord") ||
 									 IS_TOKEN(token, n, "order") ||
 									 IS_TOKEN(token, n, "sort") ||
-									 IS_TOKEN(token, n, "sortd"))
+									 IS_TOKEN(token, n, "sortd") ||
+									 IS_TOKEN(token, n, "dsort") ||
+									 IS_TOKEN(token, n, "rsort") ||
+									 IS_TOKEN(token, n, "rs"))
 							{
 								return rl_completion_matches(text, tablename_generator);
 							}
@@ -434,10 +476,10 @@ pspg_display_match(char **matches, int num_matches, int max_length)
 		c = getch();
 		echo();
 
-		if (c == 10)
+		if (c == 13)
 		{
 			rl_insert_text(matches[pos] + common_length);
-			ungetch(10);
+			ungetch(13);
 			break;
 		}
 		else if (c == 7)
@@ -502,6 +544,23 @@ get_string(char *prompt,
 	mmask_t		prev_mousemask = 0;
 	bool	prev_xterm_mouse_mode;
 
+#ifdef HAVE_READLINE_HISTORY
+
+	/*
+	 * Lazy history loading. Loading history can be slow, so use it, only
+	 * when string is wanted.
+	 */
+	if (!history_loaded)
+	{
+		log_row("history loaded");
+		read_history(tilde(NULL, saved_histfile));
+		history_loaded = true;
+
+		last_history[0] = '\0';
+	}
+
+#endif
+
 	log_row("input string prompt - \"%s\"", prompt);
 
 	editation_completed = false;
@@ -563,8 +622,6 @@ get_string(char *prompt,
 		wrefresh(prompt_window);
 	}
 
-	wtimeout(prompt_window, 100);
-
 	while (!editation_completed)
 	{
 		do
@@ -578,10 +635,29 @@ get_string(char *prompt,
 				c = wgetch(prompt_window);
 
 			if (c == ERR && errno == EINTR)
-				goto finish_read;
+			{
+				if (handle_sigwinch)
+				{
+					handle_sigwinch = false;
 
-			if (handle_sigint)
-				goto finish_read;
+					refresh_terminal_size();
+					refresh_layout_after_terminal_resize();
+					redraw_screen();
+
+					wattron(prompt_window, prompt_window_input_attr);
+					mvwprintw(prompt_window, 0, 0, "");
+					wclrtoeol(prompt_window);
+
+					rl_forced_update_display();
+
+					wrefresh(prompt_window);
+				}
+
+				else if (handle_sigint)
+					goto finish_read;
+
+				continue;
+			}
 		}
 		while (c == ERR || c == 0);
 
@@ -687,16 +763,6 @@ finish_read:
 
 	log_row("input string - \"%s\"", buffer);
 
-	/*
-	 * Keypad is an feature of an screen, not of an window. In this routine
-	 * I read from window without keypad flag, because I redirect events to
-	 * readline. But next reading from stdin looks partialy broken (like
-	 * without keypad flag), although the reading is related to stdscr with
-	 * keypad flag. I found dirty workaround - call keypad(stdscr, TRUE)
-	 * again.
-	 */
-	keypad(stdscr, TRUE);
-
 	return result_is_ok;
 }
 
@@ -706,6 +772,14 @@ finish_read:
 void
 pspg_init_readline(const char *histfile)
 {
+	/*
+	 * Don't repeat readline initialization
+	 */
+	if (readline_initized)
+		return;
+
+	readline_initized = true;
+
 	rl_catch_signals = 0;
 	rl_catch_sigwinch = 0;
 	rl_deprep_term_function = NULL;
@@ -724,8 +798,8 @@ pspg_init_readline(const char *histfile)
 
 #ifdef HAVE_READLINE_HISTORY
 
-	read_history(tilde(NULL, histfile));
-
+	saved_histfile = histfile;
+	history_loaded = false;
 	last_history[0] = '\0';
 
 #endif
@@ -744,7 +818,7 @@ pspg_init_readline(const char *histfile)
 }
 
 void
-pspg_save_history(const char *histfile)
+pspg_save_history(const char *histfile, Options *opts)
 {
 	return;
 }
@@ -803,16 +877,6 @@ get_string(char *prompt,
 
 	log_row("input string - \"%s\"", buffer);
 
-	/*
-	 * Keypad is an feature of an screen, not of an window. In this routine
-	 * I read from window without keypad flag, because I redirect events to
-	 * readline. But next reading from stdin looks partialy broken (like
-	 * without keypad flag), although the reading is related to stdscr with
-	 * keypad flag. I found dirty workaround - call keypad(stdscr, TRUE)
-	 * again.
-	 */
-	keypad(stdscr, TRUE);
-
 	return true;
 }
 
