@@ -16,6 +16,17 @@
 
 #include "pspg.h"
 
+static inline int
+signof(double n)
+{
+	if (n < 0)
+		return -1;
+	else if (n > 0)
+		return 1;
+	else
+		return 0;
+}
+
 static int
 compar_num_asc(const void *a, const void *b)
 {
@@ -25,7 +36,12 @@ compar_num_asc(const void *a, const void *b)
 	if (sdb->info == INFO_DOUBLE)
 	{
 		if (sda->info == INFO_DOUBLE)
-			return sda->d - sdb->d;
+			/*
+			 * Without using signof function there is possible problem with
+			 * mapping of 8bytes double to 4bytes int (the sign of result value
+			 * after casting can be wrong).
+			 */
+			return signof(sda->d - sdb->d);
 		else
 			return 1;
 	}
@@ -47,7 +63,7 @@ compar_num_desc(const void *a, const void *b)
 	if (sdb->info == INFO_DOUBLE)
 	{
 		if (sda->info == INFO_DOUBLE)
-			return sdb->d - sda->d;
+			return signof(sdb->d - sda->d);
 		else
 			return 1;
 	}
