@@ -149,6 +149,8 @@ extern void st_menu_free(struct ST_MENU *menu);
 extern void st_menu_save(struct ST_MENU *menu, int *cursor_rows, int **refvals, int max_items);
 extern void st_menu_load(struct ST_MENU *menu, int *cursor_rows, int **refvals);
 
+extern int st_menu_get_focus(struct ST_MENU *menu);
+
 extern ST_MENU_ITEM *st_menu_selected_item(bool *activated);
 extern ST_CMDBAR_ITEM *st_menu_selected_command(bool *activated);
 
@@ -159,6 +161,8 @@ extern bool st_menu_reset_all_submenu_options(struct ST_MENU *menu, int menu_cod
 extern bool st_menu_reset_all_options(struct ST_MENU *menu, int option);
 extern void st_menu_set_focus(struct ST_MENU *menu, int focus);
 
+extern bool st_menu_set_shortcut(struct ST_MENU *menu, int code, char *shortcut);
+
 extern bool st_menu_set_ref_option(struct ST_MENU *menu, int code, int option, int *refvalue);
 
 extern struct ST_CMDBAR *st_cmdbar_new(ST_MENU_CONFIG *config, ST_CMDBAR_ITEM *cmdbar_items);
